package com.persagy.calendar.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.persagy.calendar.constant.WorkCalendarConstant;
import com.persagy.calendar.handle.WorkCalendarDateHandler;
import com.persagy.calendar.pojo.dto.WorkCalendarDict;
import com.persagy.calendar.pojo.vo.DictCodeCreate;
import com.persagy.calendar.pojo.vo.DictCreateVO;
import com.persagy.calendar.pojo.vo.DictQueryVO;
import com.persagy.calendar.pojo.vo.WorkCalendarDateQueryVO;
import com.persagy.calendar.service.IWorkCalendarDictService;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseUtil;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 通用接口
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2020-10-03 13:54:22
 */
@Api(tags = "通用接口")
@RestController
@RequestMapping(value = "/v2.0/common", method = RequestMethod.POST)
public class CommonController {

	@Autowired
	private IWorkCalendarDictService workCalendarDictService;
	
	@Autowired
	private WorkCalendarDateHandler workCalendarDateHandler;
	
	/**
     * 手动解析规则信息
     * 非自定义已存在的作息时间信息被更新，不存在的被插入，自定义的不动
     */
    @ApiOperation(value = "立即解析规则信息")
    @RequestMapping(value = "handleParseCalendarRule")
    public ResponseResult handleParseCalendarRule(@RequestBody @Validated WorkCalendarDateQueryVO queryVO) {
    	if ("pj1111111111".equals(queryVO.getProjectId())) {
    		queryVO.setProjectId(null);
		}
    	boolean result = this.workCalendarDateHandler.handleParseCalendarRule(queryVO);
		return result ? ResponseUtil.successResult("执行成功") : ResponseUtil.errorResult("执行失败");
    }
    
	/**
     * 字典值查询
     */
    @ApiOperation(value = "字典查询")
    @RequestMapping(value = "dictQuery")
    public ResponseResult dictQuery(@RequestBody @Validated DictQueryVO queryVO) {
    	QueryWrapper<WorkCalendarDict> queryWrapper = new WorkCalendarDict.Builder().createQueryWrapper()
    			.groupCodeEq(queryVO.getGroupCode()).calendarIdEq(queryVO.getCalendarId())
    			.dictTypeEq(queryVO.getDictType()).builderQueryWrapper();
    	List<WorkCalendarDict> list = this.workCalendarDictService.list(queryWrapper);
    	if (CollectionUtil.isEmpty(list)) {
			return ResponseUtil.successResult(Lists.newArrayList());
		}
    	
    	//Map<String, Map<String, String>> dictTypeMap = new HashMap<String, Map<String, String>>(2);
    	Map<String, String> dictTypeCode = new HashMap<String, String>(8);
    	String dictType = null;
    	for (WorkCalendarDict dict : list) {
    		dictType = dict.getDictType();
    		if (WorkCalendarConstant.CUSTOM_CALENDAR_DICT_TYPE.equals(dictType)) {
    			dictTypeCode.put(StringUtil.isBlank(dict.getDictCode()) ? dict.getDictDesc() : dict.getDictCode(), dict.getDictDesc());
			} else {
				dictTypeCode.put(dict.getDictCode(), dict.getDictDesc());
			}
		}
    	//dictTypeMap.put(dictType, dictTypeCode);
		return ResponseUtil.successResult(dictTypeCode);
    }
    
    @ApiOperation(value = "字典值创建")
    @RequestMapping(value = "dictCreate")
    public ResponseResult dictCreate(@RequestBody @Validated DictCreateVO<DictCodeCreate> createVO) {
    	WorkCalendarDict calendarDict = new WorkCalendarDict();
    	BeanUtils.copyProperties(createVO, calendarDict);
    	boolean result = this.workCalendarDictService.batchCreateOrUpdateDict(calendarDict, createVO.getCodes());
    	return result ? ResponseUtil.successResult("创建成功") : ResponseUtil.successResult("创建失败");
    }
    
}
