package com.persagy.calendar.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.persagy.calendar.constant.WorkCalendarConstant;
import com.persagy.calendar.mapper.WorkCalendarRuleMapper;
import com.persagy.calendar.pojo.dto.WorkCalendarRule;
import com.persagy.calendar.pojo.dto.WorkCalendarRule.Builder;
import com.persagy.calendar.service.IWorkCalendarRuleService;
import com.persagy.common.utils.StringUtil;
import com.persagy.db.service.impl.SuperServiceImpl;

import cn.hutool.core.collection.CollectionUtil;

/**
 * 工作历规则表
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2020-10-02 18:26:14
 */
@Service
public class WorkCalendarRuleServiceImpl extends SuperServiceImpl<WorkCalendarRuleMapper, WorkCalendarRule> implements IWorkCalendarRuleService {
	
	@Autowired
	private WorkCalendarRuleMapper workCalendarRuleMapper;
 	
	@Override
	public List<WorkCalendarRule> queryRuleByCalendarId(String calendarId, String projectId) {
    	QueryWrapper<WorkCalendarRule> queryWrapper = new WorkCalendarRule.Builder().createQueryWrapper()
    			.calendarIdEq(calendarId).projectIdEq(projectId)
    			.delFlagEq(WorkCalendarConstant.WORK_CALENDAR_DEL_FLAG_NOR).builderQueryWrapper();
		return workCalendarRuleMapper.selectList(queryWrapper);
	}
	
	@Override
	public String getCalendarRule(WorkCalendarRule workCalendarRule) {
		QueryWrapper<WorkCalendarRule> queryWrapper = new WorkCalendarRule.Builder().createQueryWrapper()
    			.projectIdEq(workCalendarRule.getProjectId()).dictTypeEq(workCalendarRule.getDictType())
    			.dictCodeEq(workCalendarRule.getDictCode()).delFlagEq(WorkCalendarConstant.WORK_CALENDAR_DEL_FLAG_NOR)
    			.builderQueryWrapper();
		List<WorkCalendarRule> selectList = workCalendarRuleMapper.selectList(queryWrapper);
		if (CollectionUtil.isEmpty(selectList)) {
			return null;
		}
		for (WorkCalendarRule rule : selectList) {
			if (workCalendarRule.getValue().equals(rule.getValue())) {
				return rule.getId();
			}
		}
		return null;
	}
	
	@Override
	public WorkCalendarRule getById(String id, String projectId) {
    	QueryWrapper<WorkCalendarRule> queryWrapper = new WorkCalendarRule.Builder().createQueryWrapper()
    			.idEq(id).projectIdEq(projectId)
    			.delFlagEq(WorkCalendarConstant.WORK_CALENDAR_DEL_FLAG_NOR).builderQueryWrapper();
		return workCalendarRuleMapper.selectOne(queryWrapper);
	}
	
	@Override
	@Transactional
	public boolean createWorkCalendarRule(WorkCalendarRule workCalendarRule) {
		Integer result = workCalendarRuleMapper.insert(workCalendarRule);
		return SqlHelper.retBool(result);
	}
	
	@Override
	@Transactional
	public boolean updateWorkCalendarRule(WorkCalendarRule workCalendarRule) {
		Integer result = workCalendarRuleMapper.updateById(workCalendarRule);
		return SqlHelper.retBool(result);
	}
	
	@Override
	@Transactional
	public boolean deleteWorkCalendarRule(WorkCalendarRule workCalendarRule) {
		workCalendarRule.setDelFlag(WorkCalendarConstant.WORK_CALENDAR_DEL_FLAG_DEL);
		Integer result = null;
		if (StringUtil.isNotBlank(workCalendarRule.getId())) {
			result = workCalendarRuleMapper.updateById(workCalendarRule);
		} else {
			Builder builder = new WorkCalendarRule.Builder().createUpdateWrapper();
	    	UpdateWrapper<WorkCalendarRule> updateWrapper = builder
	    			.calendarIdEq(workCalendarRule.getCalendarId())
	    			.delFlagEq(WorkCalendarConstant.WORK_CALENDAR_DEL_FLAG_NOR).builderUpdateWrapper();
			result = workCalendarRuleMapper.update(workCalendarRule, updateWrapper);
		}
		return SqlHelper.retBool(result);
	}
	
	@Override
	@Transactional
	public boolean removeWorkCalendarRule(String id, String calendarId, String dictType) {
		Integer result = null;
		UpdateWrapper<WorkCalendarRule> updateWrapper = null;
		Builder builder = new WorkCalendarRule.Builder().createUpdateWrapper();
		if (StringUtil.isNotBlank(id)) {
			updateWrapper = builder.idEq(id).delFlagEq(WorkCalendarConstant.WORK_CALENDAR_DEL_FLAG_NOR)
					.builderUpdateWrapper();
			result = workCalendarRuleMapper.delete(updateWrapper);
		} else {
	    	updateWrapper = builder.calendarIdEq(calendarId).dictTypeEq(dictType)
	    			.delFlagEq(WorkCalendarConstant.WORK_CALENDAR_DEL_FLAG_NOR).builderUpdateWrapper();
			result = workCalendarRuleMapper.delete(updateWrapper);
		}
		return SqlHelper.retBool(result);
	}

	@Override
	public boolean checkDictRuleValid(String projectId, String calendarId, String dictType, String dictCode, String id) {
		QueryWrapper<WorkCalendarRule> queryWrapper = new WorkCalendarRule.Builder().createQueryWrapper()
    			.projectIdEq(projectId).calendarIdEq(calendarId).dictTypeEq(dictType).dictCodeEq(dictCode)
    			.delFlagEq(WorkCalendarConstant.WORK_CALENDAR_DEL_FLAG_NOR).builderQueryWrapper();
		WorkCalendarRule calendarRule = this.workCalendarRuleMapper.selectOne(queryWrapper);
		if (calendarRule == null) {
			return true;
		}
		if (StringUtil.isNotBlank(id) && id.equals(calendarRule.getId())) {
			return true;
		}
		return false;
	}

}
